import "Turbine"
import "Turbine.UI"

local width, height = Turbine.UI.Display.GetSize()

_G.Grid   = _G.Grid or {}
_G.Grid.W = _G.Grid.W or {}
_G.Grid.H = _G.Grid.H or {}

_G.Grid.Guide   = false
_G.Grid.Depth   = 0
_G.Grid.Alpha   = 0.25
_G.Grid.Visible = false

_G.Grid.X = _G.Grid.X or Turbine.UI.Window()
_G.Grid.Y = _G.Grid.Y or Turbine.UI.Window()

_G.Grid.GX = _G.Grid.GX or Turbine.UI.Window()
_G.Grid.GY = _G.Grid.GY or Turbine.UI.Window()

local gridSize = 12

_G.Grid.CountW = ( width / gridSize ) + 1
_G.Grid.CountH = ( height / gridSize ) + 1

for i = 1, _G.Grid.CountW do
    _G.Grid.W[ i ] = Turbine.UI.Window()

    local tbl = _G.Grid.W[ i ]

    tbl:SetPosition( gridSize * i - 1 - gridSize, 0 )
    tbl:SetSize( 2, height )
    tbl:SetBackColor( Turbine.UI.Color( _G.Grid.Alpha, 1, 1, 1 ) )
    tbl:SetZOrder( -3 )
    tbl:SetMouseVisible( false )
end

for i = 1, _G.Grid.CountH do
    _G.Grid.H[ i ] = Turbine.UI.Window()

    local tbl = _G.Grid.H[ i ]

    tbl:SetPosition( 0, gridSize * i - 1 - gridSize )
    tbl:SetSize( width, 2 )
    tbl:SetBackColor( Turbine.UI.Color( _G.Grid.Alpha, 1, 1, 1 ) )
    tbl:SetZOrder( -3 )
    tbl:SetMouseVisible( false )
end

_G.Grid.X:SetPosition( 0, height / 2 - 1 )
_G.Grid.X:SetSize( width, 2 )
_G.Grid.X:SetBackColor( Turbine.UI.Color( 1, 0, 0 ) )
_G.Grid.X:SetZOrder( -2 )
_G.Grid.X:SetMouseVisible( false )

_G.Grid.Y:SetPosition( width / 2 - 1, 0 )
_G.Grid.Y:SetSize( 2, height )
_G.Grid.Y:SetBackColor( Turbine.UI.Color( 1, 0, 0 ) )
_G.Grid.Y:SetZOrder( -2 )
_G.Grid.Y:SetMouseVisible( false )

_G.Grid.GX:SetPosition( 0, 0 )
_G.Grid.GX:SetSize( width, 1 )
_G.Grid.GX:SetBackColor( Turbine.UI.Color( 0, 1, 0 ) )
_G.Grid.GX:SetZOrder( -1 )
_G.Grid.GX:SetMouseVisible( false )

_G.Grid.GY:SetPosition( 0, 0 )
_G.Grid.GY:SetSize( 1, height )
_G.Grid.GY:SetBackColor( Turbine.UI.Color( 0, 1, 0 ) )
_G.Grid.GY:SetZOrder( -1 )
_G.Grid.GY:SetMouseVisible( false )

function toggleGrid( bool )
    _G.Grid.Visible = bool

    for i = 1, _G.Grid.CountW do
        _G.Grid.W[ i ]:SetVisible( bool )
    end

    for i = 1, _G.Grid.CountH do
        _G.Grid.H[ i ]:SetVisible( bool )
    end

    _G.Grid.X:SetVisible( bool )
    _G.Grid.Y:SetVisible( bool )

    if bool == true then
        _G.Grid.GX:SetVisible( _G.Grid.Guide )
        _G.Grid.GY:SetVisible( _G.Grid.Guide )
    else
        _G.Grid.GX:SetVisible( bool )
        _G.Grid.GY:SetVisible( bool )
    end
end

function toggleGuide( bool )
    _G.Grid.GX:SetVisible( bool )
    _G.Grid.GY:SetVisible( bool )

    _G.Grid.Guide = bool
end

function setDepth( depth )
    _G.Grid.Depth = depth

    for i = 1, _G.Grid.CountW do
        _G.Grid.W[ i ]:SetZOrder( -3 + depth )
    end

    for i = 1, _G.Grid.CountH do
        _G.Grid.H[ i ]:SetZOrder( -3 + depth )
    end

    _G.Grid.X:SetZOrder( -2 + depth )
    _G.Grid.Y:SetZOrder( -2 + depth )

    _G.Grid.GX:SetZOrder( -1 + depth )
    _G.Grid.GY:SetZOrder( -1 + depth )
end

function setSize( size )
    for i = 1, _G.Grid.CountW do
        _G.Grid.W[ i ]:SetVisible( false )
    end

    for i = 1, _G.Grid.CountH do
        _G.Grid.H[ i ]:SetVisible( false )
    end

    _G.Grid.W = {}
    _G.Grid.H = {}

    collectgarbage()

    _G.Grid.CountW = ( width / size ) + 1
    _G.Grid.CountH = ( height / size ) + 1

    for i = 1, _G.Grid.CountW do
        _G.Grid.W[ i ] = Turbine.UI.Window()

        local tbl = _G.Grid.W[ i ]

        tbl:SetPosition( size * i - 1 - size, 0 )
        tbl:SetSize( 2, height )
        tbl:SetBackColor( Turbine.UI.Color( _G.Grid.Alpha, 1, 1, 1 ) )
        tbl:SetZOrder( -3 + _G.Grid.Depth )
        tbl:SetMouseVisible( false )
        tbl:SetVisible( _G.Grid.Visible )
    end

    for i = 1, _G.Grid.CountH do
        _G.Grid.H[ i ] = Turbine.UI.Window()

        local tbl = _G.Grid.H[ i ]

        tbl:SetPosition( 0, size * i - 1 - size )
        tbl:SetSize( width, 2 )
        tbl:SetBackColor( Turbine.UI.Color( _G.Grid.Alpha, 1, 1, 1 ) )
        tbl:SetZOrder( -3 + _G.Grid.Depth )
        tbl:SetMouseVisible( false )
        tbl:SetVisible( _G.Grid.Visible )
    end
end

command = class( Turbine.ShellCommand )

function command:Execute( _, arguments )
    if arguments == nil or string.len( arguments ) == 0 then
        Turbine.Shell.WriteLine( self:Help() )
        return
    end

    local argumentList = { }
    local index = 1

    for word in arguments:gmatch( "%w+" ) do
        argumentList[ index ] = word
        index = index + 1
    end

    local cmd = string.lower( argumentList[ 1 ] )

    if cmd == "show" then
        toggleGrid( true )
    elseif cmd == "hide" then
        toggleGrid( false )
    elseif cmd == "foreground" then
        setDepth( 100 )
    elseif cmd == "background" then
        setDepth( 0 )
    elseif cmd == "size" then
        local size = argumentList[ 2 ]

        if size ~= nil and string.len( size ) > 0 then
            size = tonumber( size )

            if size ~= nil then
                setSize( size )
            end
        end
    elseif cmd == "alpha" then
        local alpha = argumentList[ 2 ]

        if alpha ~= nil and string.len( alpha ) > 0 then
            alpha = tonumber( alpha )

            if alpha ~= nil then
                _G.Grid.Alpha = 1 / 255 * alpha

                for i = 1, widthCount do
                    _G.Grid.W[ i ]:SetBackColor( Turbine.UI.Color( 1 / 255 * alpha, 1, 1, 1 ) )
                end

                for i = 1, heightCount do
                    _G.Grid.H[ i ]:SetBackColor( Turbine.UI.Color( 1 / 255 * alpha, 1, 1, 1 ) )
                end
            end
        end
    elseif cmd == "guide" then
        if argumentList[ 2 ] == "show" then
            toggleGuide( true )
        elseif argumentList[ 2 ] == "hide" then
            toggleGuide( false )
        elseif argumentList[ 2 ] == "size" then
            local size = argumentList[ 3 ]

            if size ~= nil and string.len( size ) > 0 then
                size = tonumber( size )

                if size ~= nil then
                    _G.Grid.GX:SetSize( width, size )
                    _G.Grid.GY:SetSize( size, height )
                end
            end
        else
            if not _G.Grid.Guide then return end

            local x = argumentList[ 2 ]
            local y = argumentList[ 3 ]

            if x ~= nil and string.len( x ) > 0 and y ~= nil and string.len( y ) > 0 then
                x = tonumber( x )
                y = tonumber( y )

                if x ~= nil and y ~= nil then
                    _G.Grid.GX:SetPosition( 0, x )
                    _G.Grid.GY:SetPosition( y, 0 )
                end
            end
        end
    elseif cmd == "help" then
        Turbine.Shell.WriteLine( self:Help() )
    else
        Turbine.Shell.WriteLine( "Invalid command: " .. cmd )
        Turbine.Shell.WriteLine( self:Help() )
    end
end

function command:Help()
    local str = "\n________________\n/grid [ show | hide ]\n"
    str = str .. "/grid size [ number ]\n"
    str = str .. "/grid [ foreground | background ]\n"
    str = str .. "/grid alpha [ 0 - 255 ]\n\n"
    str = str .. "/grid guide [ show | hide ]\n"
    str = str .. "/grid guide size [ number ]\n"
    str = str .. "/grid guide [ x y ]\n"

    return str
end

command = command()

Turbine.Shell.AddCommand( "grid", command )
